# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .get_report_runs_data_object_response_body_status import GetReportRunsDataObjectResponseBodyStatus
from .report_columns_object_response_body import ReportColumnsObjectResponseBody


class GetReportRunsDataObjectResponseBody(UniversalBaseModel):
    columns: typing.List[ReportColumnsObjectResponseBody] = pydantic.Field()
    """
    Columns of this report run.
    """

    rows: typing.List[typing.List[typing.Dict[str, typing.Any]]] = pydantic.Field()
    """
    Array where each inner array represents a row of data, and each element in the inner array is a value for the column. Inner array items are nullable, data types: double, string, timestamp as string in RFC3339 format. The number of columns defined in the report run is to the number of inner array items.
    """

    status: GetReportRunsDataObjectResponseBodyStatus = pydantic.Field()
    """
    Status of the report run.  Valid values: `complete`, `pending`, `failed`, `canceled`, `unknown`
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..errors.bad_gateway_error import BadGatewayError
from ..errors.gateway_timeout_error import GatewayTimeoutError
from ..errors.internal_server_error import InternalServerError
from ..errors.method_not_allowed_error import MethodNotAllowedError
from ..errors.not_found_error import NotFoundError
from ..errors.not_implemented_error import NotImplementedError
from ..errors.service_unavailable_error import ServiceUnavailableError
from ..errors.too_many_requests_error import TooManyRequestsError
from ..errors.unauthorized_error import UnauthorizedError
from ..types.safety_events_v_2_get_safety_events_v_2_response_body import SafetyEventsV2GetSafetyEventsV2ResponseBody
from ..types.safety_events_v_2_get_safety_events_v_2_stream_response_body import (
    SafetyEventsV2GetSafetyEventsV2StreamResponseBody,
)
from ..types.v_1_driver_safety_score_response import V1DriverSafetyScoreResponse
from ..types.v_1_vehicle_safety_score_response import V1VehicleSafetyScoreResponse
from .types.get_safety_events_v_2_stream_request_query_by_time_field import (
    GetSafetyEventsV2StreamRequestQueryByTimeField,
)


class RawSafetyClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get_safety_events_v_2(
        self,
        *,
        safety_event_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        include_asset: typing.Optional[bool] = None,
        include_driver: typing.Optional[bool] = None,
        include_vg_only_events: typing.Optional[bool] = None,
        after: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[SafetyEventsV2GetSafetyEventsV2ResponseBody]:
        """
        This endpoint will return details for the specified safety events based on the parameters passed in. Results are paginated.

         <b>Rate limit:</b> 5 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Read Safety Events & Scores** under the Safety & Cameras category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        safety_event_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Required string of comma separated Safety Event IDs. Unique Samsara IDs (uuid) of the safety event.

        include_asset : typing.Optional[bool]
            Indicates whether or not to return expanded “asset” data

        include_driver : typing.Optional[bool]
            Indicates whether or not to return expanded “driver” data

        include_vg_only_events : typing.Optional[bool]
            Indicates whether or not to return events that are captured by devices with only a Vehicle Gateway (VG) installed.

        after : typing.Optional[str]
             If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[SafetyEventsV2GetSafetyEventsV2ResponseBody]
            OK response.
        """
        _response = self._client_wrapper.httpx_client.request(
            "safety-events",
            method="GET",
            params={
                "safetyEventIds": safety_event_ids,
                "includeAsset": include_asset,
                "includeDriver": include_driver,
                "includeVgOnlyEvents": include_vg_only_events,
                "after": after,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SafetyEventsV2GetSafetyEventsV2ResponseBody,
                    parse_obj_as(
                        type_=SafetyEventsV2GetSafetyEventsV2ResponseBody,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_safety_events_v_2_stream(
        self,
        *,
        start_time: str,
        end_time: typing.Optional[str] = None,
        query_by_time_field: typing.Optional[GetSafetyEventsV2StreamRequestQueryByTimeField] = None,
        asset_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        driver_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        tag_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        assigned_coaches: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        behavior_labels: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        event_states: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        include_asset: typing.Optional[bool] = None,
        include_driver: typing.Optional[bool] = None,
        include_vg_only_events: typing.Optional[bool] = None,
        after: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[SafetyEventsV2GetSafetyEventsV2StreamResponseBody]:
        """
        This endpoint will return all safety events associated with your organization based on the parameters passed in. To get core endpoint data, select Read Safety Events & Scores under the Safety & Cameras category when creating or editing an API token. Read Camera Media permissions required to get Safety Event video media via this endpoint. If you include an endTime, the endpoint will return data up until that point. If you do not include an endTime, you can continue to poll the API real-time with the pagination cursor that gets returned on every call. Results are paginated.

         <b>Rate limit:</b> 5 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Read Safety Events & Scores** under the Safety & Cameras category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        start_time : str
            RFC 3339 timestamp that indicates when to begin receiving data. Value is compared against `updatedAtTime` or `createdAtTime` depending on the `queryByTimeField` parameter.

        end_time : typing.Optional[str]
            RFC 3339 timestamp. If not provided and filtering by `updatedAtTime` then the endpoint behaves as an unending feed of changes. If endTime is set the same as startTime, the most recent data point before that time will be returned per asset. Value is compared against `updatedAtTime` or `createdAtTime` depending on the `queryByTimeField` parameter.

        query_by_time_field : typing.Optional[GetSafetyEventsV2StreamRequestQueryByTimeField]
            Optional string that decides which field to compare against the provided time range.  Valid values: `updatedAtTime`, `createdAtTime`

        asset_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Optional string of comma separated asset IDs. If asset ID is present, events for the specified asset(s) will be returned. Limit of 2000 asset IDs.

        driver_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Optional string of comma separated driver IDs. If driver ID is present, events for the specified driver(s) will be returned. Limit of 2000 driver IDs.

        tag_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Optional string of comma separated tag IDs. If tag ID is present, events for the specified tag(s) will be returned. Limit of 2000 tag IDs.

        assigned_coaches : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Optional string of comma separated coach IDs to filter events assigned to a particular coach. Limit of 2000 coach IDs.

        behavior_labels : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Optional string of comma separated values to filter behavior labels. Valid values: `Acceleration`, `AggressiveDriving`, `BluetoothHeadset`, `Braking`, `ContextConstructionOrWorkZone`, `ContextSnowyOrIcy`, `ContextVulnerableRoadUser`, `ContextWet`, `Crash`, `DefensiveDriving`, `DidNotYield`, `Drinking`, `Drowsy`, `Eating`, `EatingDrinking`, `EdgeDistractedDriving`, `EdgeRailroadCrossingViolation`, `FollowingDistance`, `FollowingDistanceModerate`, `FollowingDistanceSevere`, `ForwardCollisionWarning`, `GenericDistraction`, `GenericTailgating`, `HarshTurn`, `HeavySpeeding`, `HosViolation`, `Idling`, `Invalid`, `LaneDeparture`, `LateResponse`, `LeftTurn`, `LightSpeeding`, `MaxSpeed`, `MobileUsage`, `ModerateSpeeding`, `NearCollison`, `NearPedestrianCollision`, `NoSeatbelt`, `ObstructedCamera`, `OtherViolation`, `Passenger`, `PolicyViolationMask`, `ProtectiveEquipment`, `RanRedLight`, `Reversing`, `RollingStop`, `RolloverProtection`, `SevereSpeeding`, `Smoking`, `Speeding`, `UTurn`, `UnsafeManeuver`, `UnsafeParking`, `VulnerableRoadUserCollisionWarning`, `YawControl`

        event_states : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Optional string of comma separated values to filter event states. Valid values: `needsReview`, `reviewed`, `needsCoaching`, `coached`, `dismissed`, `needsRecognition`, `recognized`

        include_asset : typing.Optional[bool]
            Indicates whether or not to return expanded “asset” data

        include_driver : typing.Optional[bool]
            Indicates whether or not to return expanded “driver” data

        include_vg_only_events : typing.Optional[bool]
            Indicates whether or not to return events that are captured by devices with only a Vehicle Gateway (VG) installed.

        after : typing.Optional[str]
             If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[SafetyEventsV2GetSafetyEventsV2StreamResponseBody]
            OK response.
        """
        _response = self._client_wrapper.httpx_client.request(
            "safety-events/stream",
            method="GET",
            params={
                "startTime": start_time,
                "endTime": end_time,
                "queryByTimeField": query_by_time_field,
                "assetIds": asset_ids,
                "driverIds": driver_ids,
                "tagIds": tag_ids,
                "assignedCoaches": assigned_coaches,
                "behaviorLabels": behavior_labels,
                "eventStates": event_states,
                "includeAsset": include_asset,
                "includeDriver": include_driver,
                "includeVgOnlyEvents": include_vg_only_events,
                "after": after,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SafetyEventsV2GetSafetyEventsV2StreamResponseBody,
                    parse_obj_as(
                        type_=SafetyEventsV2GetSafetyEventsV2StreamResponseBody,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def v_1_get_driver_safety_score(
        self, driver_id: int, *, start_ms: int, end_ms: int, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[V1DriverSafetyScoreResponse]:
        """
        <n class="warning">
        <nh>
        <i class="fa fa-exclamation-circle"></i>
        This endpoint is still on our legacy API.
        </nh>
        </n>

        Fetch the safety score for the driver.

         <b>Rate limit:</b> 5 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Read Safety Events & Scores** under the Safety & Cameras category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        driver_id : int
            ID of the driver. Must contain only digits 0-9.

        start_ms : int
            Timestamp in milliseconds representing the start of the period to fetch, inclusive. Used in combination with endMs. Total duration (endMs - startMs) must be greater than or equal to 1 hour.

        end_ms : int
            Timestamp in milliseconds representing the end of the period to fetch, inclusive. Used in combination with startMs. Total duration (endMs - startMs) must be greater than or equal to 1 hour.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[V1DriverSafetyScoreResponse]
            Safety score details.
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/fleet/drivers/{jsonable_encoder(driver_id)}/safety/score",
            method="GET",
            params={
                "startMs": start_ms,
                "endMs": end_ms,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    V1DriverSafetyScoreResponse,
                    parse_obj_as(
                        type_=V1DriverSafetyScoreResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def v_1_get_vehicle_safety_score(
        self, vehicle_id: int, *, start_ms: int, end_ms: int, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[V1VehicleSafetyScoreResponse]:
        """
        <n class="warning">
        <nh>
        <i class="fa fa-exclamation-circle"></i>
        This endpoint is still on our legacy API.
        </nh>
        </n>

        Fetch the safety score for the vehicle.

         <b>Rate limit:</b> 5 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Read Safety Events & Scores** under the Safety & Cameras category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        vehicle_id : int
            ID of the vehicle. Must contain only digits 0-9.

        start_ms : int
            Timestamp in milliseconds representing the start of the period to fetch, inclusive. Used in combination with endMs. Total duration (endMs - startMs) must be greater than or equal to 1 hour.

        end_ms : int
            Timestamp in milliseconds representing the end of the period to fetch, inclusive. Used in combination with startMs. Total duration (endMs - startMs) must be greater than or equal to 1 hour.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[V1VehicleSafetyScoreResponse]
            Safety score details.
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/fleet/vehicles/{jsonable_encoder(vehicle_id)}/safety/score",
            method="GET",
            params={
                "startMs": start_ms,
                "endMs": end_ms,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    V1VehicleSafetyScoreResponse,
                    parse_obj_as(
                        type_=V1VehicleSafetyScoreResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawSafetyClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get_safety_events_v_2(
        self,
        *,
        safety_event_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        include_asset: typing.Optional[bool] = None,
        include_driver: typing.Optional[bool] = None,
        include_vg_only_events: typing.Optional[bool] = None,
        after: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[SafetyEventsV2GetSafetyEventsV2ResponseBody]:
        """
        This endpoint will return details for the specified safety events based on the parameters passed in. Results are paginated.

         <b>Rate limit:</b> 5 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Read Safety Events & Scores** under the Safety & Cameras category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        safety_event_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Required string of comma separated Safety Event IDs. Unique Samsara IDs (uuid) of the safety event.

        include_asset : typing.Optional[bool]
            Indicates whether or not to return expanded “asset” data

        include_driver : typing.Optional[bool]
            Indicates whether or not to return expanded “driver” data

        include_vg_only_events : typing.Optional[bool]
            Indicates whether or not to return events that are captured by devices with only a Vehicle Gateway (VG) installed.

        after : typing.Optional[str]
             If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[SafetyEventsV2GetSafetyEventsV2ResponseBody]
            OK response.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "safety-events",
            method="GET",
            params={
                "safetyEventIds": safety_event_ids,
                "includeAsset": include_asset,
                "includeDriver": include_driver,
                "includeVgOnlyEvents": include_vg_only_events,
                "after": after,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SafetyEventsV2GetSafetyEventsV2ResponseBody,
                    parse_obj_as(
                        type_=SafetyEventsV2GetSafetyEventsV2ResponseBody,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_safety_events_v_2_stream(
        self,
        *,
        start_time: str,
        end_time: typing.Optional[str] = None,
        query_by_time_field: typing.Optional[GetSafetyEventsV2StreamRequestQueryByTimeField] = None,
        asset_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        driver_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        tag_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        assigned_coaches: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        behavior_labels: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        event_states: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        include_asset: typing.Optional[bool] = None,
        include_driver: typing.Optional[bool] = None,
        include_vg_only_events: typing.Optional[bool] = None,
        after: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[SafetyEventsV2GetSafetyEventsV2StreamResponseBody]:
        """
        This endpoint will return all safety events associated with your organization based on the parameters passed in. To get core endpoint data, select Read Safety Events & Scores under the Safety & Cameras category when creating or editing an API token. Read Camera Media permissions required to get Safety Event video media via this endpoint. If you include an endTime, the endpoint will return data up until that point. If you do not include an endTime, you can continue to poll the API real-time with the pagination cursor that gets returned on every call. Results are paginated.

         <b>Rate limit:</b> 5 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Read Safety Events & Scores** under the Safety & Cameras category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        start_time : str
            RFC 3339 timestamp that indicates when to begin receiving data. Value is compared against `updatedAtTime` or `createdAtTime` depending on the `queryByTimeField` parameter.

        end_time : typing.Optional[str]
            RFC 3339 timestamp. If not provided and filtering by `updatedAtTime` then the endpoint behaves as an unending feed of changes. If endTime is set the same as startTime, the most recent data point before that time will be returned per asset. Value is compared against `updatedAtTime` or `createdAtTime` depending on the `queryByTimeField` parameter.

        query_by_time_field : typing.Optional[GetSafetyEventsV2StreamRequestQueryByTimeField]
            Optional string that decides which field to compare against the provided time range.  Valid values: `updatedAtTime`, `createdAtTime`

        asset_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Optional string of comma separated asset IDs. If asset ID is present, events for the specified asset(s) will be returned. Limit of 2000 asset IDs.

        driver_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Optional string of comma separated driver IDs. If driver ID is present, events for the specified driver(s) will be returned. Limit of 2000 driver IDs.

        tag_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Optional string of comma separated tag IDs. If tag ID is present, events for the specified tag(s) will be returned. Limit of 2000 tag IDs.

        assigned_coaches : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Optional string of comma separated coach IDs to filter events assigned to a particular coach. Limit of 2000 coach IDs.

        behavior_labels : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Optional string of comma separated values to filter behavior labels. Valid values: `Acceleration`, `AggressiveDriving`, `BluetoothHeadset`, `Braking`, `ContextConstructionOrWorkZone`, `ContextSnowyOrIcy`, `ContextVulnerableRoadUser`, `ContextWet`, `Crash`, `DefensiveDriving`, `DidNotYield`, `Drinking`, `Drowsy`, `Eating`, `EatingDrinking`, `EdgeDistractedDriving`, `EdgeRailroadCrossingViolation`, `FollowingDistance`, `FollowingDistanceModerate`, `FollowingDistanceSevere`, `ForwardCollisionWarning`, `GenericDistraction`, `GenericTailgating`, `HarshTurn`, `HeavySpeeding`, `HosViolation`, `Idling`, `Invalid`, `LaneDeparture`, `LateResponse`, `LeftTurn`, `LightSpeeding`, `MaxSpeed`, `MobileUsage`, `ModerateSpeeding`, `NearCollison`, `NearPedestrianCollision`, `NoSeatbelt`, `ObstructedCamera`, `OtherViolation`, `Passenger`, `PolicyViolationMask`, `ProtectiveEquipment`, `RanRedLight`, `Reversing`, `RollingStop`, `RolloverProtection`, `SevereSpeeding`, `Smoking`, `Speeding`, `UTurn`, `UnsafeManeuver`, `UnsafeParking`, `VulnerableRoadUserCollisionWarning`, `YawControl`

        event_states : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Optional string of comma separated values to filter event states. Valid values: `needsReview`, `reviewed`, `needsCoaching`, `coached`, `dismissed`, `needsRecognition`, `recognized`

        include_asset : typing.Optional[bool]
            Indicates whether or not to return expanded “asset” data

        include_driver : typing.Optional[bool]
            Indicates whether or not to return expanded “driver” data

        include_vg_only_events : typing.Optional[bool]
            Indicates whether or not to return events that are captured by devices with only a Vehicle Gateway (VG) installed.

        after : typing.Optional[str]
             If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[SafetyEventsV2GetSafetyEventsV2StreamResponseBody]
            OK response.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "safety-events/stream",
            method="GET",
            params={
                "startTime": start_time,
                "endTime": end_time,
                "queryByTimeField": query_by_time_field,
                "assetIds": asset_ids,
                "driverIds": driver_ids,
                "tagIds": tag_ids,
                "assignedCoaches": assigned_coaches,
                "behaviorLabels": behavior_labels,
                "eventStates": event_states,
                "includeAsset": include_asset,
                "includeDriver": include_driver,
                "includeVgOnlyEvents": include_vg_only_events,
                "after": after,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SafetyEventsV2GetSafetyEventsV2StreamResponseBody,
                    parse_obj_as(
                        type_=SafetyEventsV2GetSafetyEventsV2StreamResponseBody,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def v_1_get_driver_safety_score(
        self, driver_id: int, *, start_ms: int, end_ms: int, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[V1DriverSafetyScoreResponse]:
        """
        <n class="warning">
        <nh>
        <i class="fa fa-exclamation-circle"></i>
        This endpoint is still on our legacy API.
        </nh>
        </n>

        Fetch the safety score for the driver.

         <b>Rate limit:</b> 5 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Read Safety Events & Scores** under the Safety & Cameras category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        driver_id : int
            ID of the driver. Must contain only digits 0-9.

        start_ms : int
            Timestamp in milliseconds representing the start of the period to fetch, inclusive. Used in combination with endMs. Total duration (endMs - startMs) must be greater than or equal to 1 hour.

        end_ms : int
            Timestamp in milliseconds representing the end of the period to fetch, inclusive. Used in combination with startMs. Total duration (endMs - startMs) must be greater than or equal to 1 hour.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[V1DriverSafetyScoreResponse]
            Safety score details.
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/fleet/drivers/{jsonable_encoder(driver_id)}/safety/score",
            method="GET",
            params={
                "startMs": start_ms,
                "endMs": end_ms,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    V1DriverSafetyScoreResponse,
                    parse_obj_as(
                        type_=V1DriverSafetyScoreResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def v_1_get_vehicle_safety_score(
        self, vehicle_id: int, *, start_ms: int, end_ms: int, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[V1VehicleSafetyScoreResponse]:
        """
        <n class="warning">
        <nh>
        <i class="fa fa-exclamation-circle"></i>
        This endpoint is still on our legacy API.
        </nh>
        </n>

        Fetch the safety score for the vehicle.

         <b>Rate limit:</b> 5 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Read Safety Events & Scores** under the Safety & Cameras category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        vehicle_id : int
            ID of the vehicle. Must contain only digits 0-9.

        start_ms : int
            Timestamp in milliseconds representing the start of the period to fetch, inclusive. Used in combination with endMs. Total duration (endMs - startMs) must be greater than or equal to 1 hour.

        end_ms : int
            Timestamp in milliseconds representing the end of the period to fetch, inclusive. Used in combination with startMs. Total duration (endMs - startMs) must be greater than or equal to 1 hour.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[V1VehicleSafetyScoreResponse]
            Safety score details.
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/fleet/vehicles/{jsonable_encoder(vehicle_id)}/safety/score",
            method="GET",
            params={
                "startMs": start_ms,
                "endMs": end_ms,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    V1VehicleSafetyScoreResponse,
                    parse_obj_as(
                        type_=V1VehicleSafetyScoreResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .policy_violations_detection_alert_settings_object_response_body_events_available_for_testing_item import (
    PolicyViolationsDetectionAlertSettingsObjectResponseBodyEventsAvailableForTestingItem,
)
from .policy_violations_detection_alert_settings_object_response_body_events_to_coach_item import (
    PolicyViolationsDetectionAlertSettingsObjectResponseBodyEventsToCoachItem,
)


class PolicyViolationsDetectionAlertSettingsObjectResponseBody(UniversalBaseModel):
    """
    Enables detection of policy violations, surfaces events in Safety Inbox, and enables configurable alerts. While the feature is in beta, in-cab alerts are recommended for testing use only.
    """

    events_available_for_testing: typing_extensions.Annotated[
        typing.Optional[
            typing.List[PolicyViolationsDetectionAlertSettingsObjectResponseBodyEventsAvailableForTestingItem]
        ],
        FieldMetadata(alias="eventsAvailableForTesting"),
        pydantic.Field(
            alias="eventsAvailableForTesting",
            description="List of selectable beta policy violation events to be tested.",
        ),
    ] = None
    events_to_coach: typing_extensions.Annotated[
        typing.Optional[typing.List[PolicyViolationsDetectionAlertSettingsObjectResponseBodyEventsToCoachItem]],
        FieldMetadata(alias="eventsToCoach"),
        pydantic.Field(
            alias="eventsToCoach", description="List of selectable policy violation events to enable coaching for."
        ),
    ] = None
    has_in_cab_audio_alerts_enabled: typing_extensions.Annotated[
        typing.Optional[bool],
        FieldMetadata(alias="hasInCabAudioAlertsEnabled"),
        pydantic.Field(
            alias="hasInCabAudioAlertsEnabled",
            description="Indicates whether in-cab audio alerts for rolling stops are turned on.",
        ),
    ] = None
    is_enabled: typing_extensions.Annotated[
        typing.Optional[bool],
        FieldMetadata(alias="isEnabled"),
        pydantic.Field(
            alias="isEnabled", description="Indicates whether AI event detection for rolling stops is turned on."
        ),
    ] = None
    speeding_threshold_mph: typing_extensions.Annotated[
        typing.Optional[float],
        FieldMetadata(alias="speedingThresholdMph"),
        pydantic.Field(alias="speedingThresholdMph", description="Alert when speed is over this many miles per hour."),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .trip_asset_response_body_type import TripAssetResponseBodyType


class TripAssetResponseBody(UniversalBaseModel):
    """
    Asset that the location readings are tied to
    """

    id: str = pydantic.Field()
    """
    Unique ID for the asset object that is reporting the location.
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    Name for the asset object that is reporting the location. Only returns when `includeAsset` is set to `true`.
    """

    type: typing.Optional[TripAssetResponseBodyType] = pydantic.Field(default=None)
    """
    Type for the asset object that is reporting the location. Only returns when `includeAsset` is set to `true`.  Valid values: `uncategorized`, `trailer`, `equipment`, `unpowered`, `vehicle`
    """

    vin: typing.Optional[str] = pydantic.Field(default=None)
    """
    VIN for the asset object that is reporting the location. Only returns when `includeAsset` is set to `true`.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

LiveSharingLinksCreateLiveSharingLinkRequestBodyType = typing.Union[
    typing.Literal["assetsLocation", "assetsNearLocation", "assetsOnRoute"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .alert_object_sites_response_body import AlertObjectSitesResponseBody
from .goa_tag_tiny_response_response_body import GoaTagTinyResponseResponseBody


class AlertObjectWorkforceCameraDeviceResponseBody(UniversalBaseModel):
    """
    A camera device associated with the alert
    """

    id: str = pydantic.Field()
    """
    The ID of the camera device associated with the alert
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The name of the camera device
    """

    sites: typing.Optional[typing.List[AlertObjectSitesResponseBody]] = pydantic.Field(default=None)
    """
    The list of sites associated with the camera device.
    """

    tags: typing.Optional[typing.List[GoaTagTinyResponseResponseBody]] = pydantic.Field(default=None)
    """
    The list of [tags](https://kb.samsara.com/hc/en-us/articles/360026674631-Using-Tags-and-Tag-Nesting) associated with the camera device.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .harsh_accel_sensitivity_g_force_settings_object_response_body import (
    HarshAccelSensitivityGForceSettingsObjectResponseBody,
)
from .harsh_brake_sensitivity_g_force_settings_object_response_body import (
    HarshBrakeSensitivityGForceSettingsObjectResponseBody,
)
from .harsh_turn_sensitivity_g_force_settings_object_response_body import (
    HarshTurnSensitivityGForceSettingsObjectResponseBody,
)


class HarshEventSensitivitySettingsObjectResponseBody(UniversalBaseModel):
    """
    The configurable sensitivity for Harsh Event Detection on CM11/CM12/CM22 devices. Sensitivity can be measured as a numeric g-force value or the following values: `Normal`, `Less Sensitive`, `More Sensitive`.
    """

    harsh_accel_sensitivity_g_force: typing_extensions.Annotated[
        typing.Optional[HarshAccelSensitivityGForceSettingsObjectResponseBody],
        FieldMetadata(alias="harshAccelSensitivityGForce"),
        pydantic.Field(alias="harshAccelSensitivityGForce"),
    ] = None
    harsh_brake_sensitivity_g_force: typing_extensions.Annotated[
        typing.Optional[HarshBrakeSensitivityGForceSettingsObjectResponseBody],
        FieldMetadata(alias="harshBrakeSensitivityGForce"),
        pydantic.Field(alias="harshBrakeSensitivityGForce"),
    ] = None
    harsh_turn_sensitivity_g_force: typing_extensions.Annotated[
        typing.Optional[HarshTurnSensitivityGForceSettingsObjectResponseBody],
        FieldMetadata(alias="harshTurnSensitivityGForce"),
        pydantic.Field(alias="harshTurnSensitivityGForce"),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .ridership_passenger_identifier_object_response_body import RidershipPassengerIdentifierObjectResponseBody
from .ridership_passenger_special_instructions_object_response_body import (
    RidershipPassengerSpecialInstructionsObjectResponseBody,
)


class RidershipPassengerObjectResponseBody(UniversalBaseModel):
    """
    A ridership passenger entity.
    """

    account_id: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="accountId"),
        pydantic.Field(alias="accountId", description="The Samsara UUID of the account this passenger belongs to."),
    ]
    classification: typing.Optional[str] = pydantic.Field(default=None)
    """
    Classification or grade level of the passenger.
    """

    created_at_time: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="createdAtTime"),
        pydantic.Field(alias="createdAtTime", description="The time the passenger was created in RFC 3339 format."),
    ]
    external_ids: typing_extensions.Annotated[
        typing.Optional[typing.Dict[str, str]],
        FieldMetadata(alias="externalIds"),
        pydantic.Field(alias="externalIds", description="A map of external ids"),
    ] = None
    first_name: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="firstName"),
        pydantic.Field(alias="firstName", description="First name of the passenger."),
    ]
    id: str = pydantic.Field()
    """
    The unique Samsara ID of the passenger.
    """

    identifiers: typing.Optional[typing.List[RidershipPassengerIdentifierObjectResponseBody]] = pydantic.Field(
        default=None
    )
    """
    List of identifiers associated with the passenger.
    """

    is_active: typing_extensions.Annotated[
        bool,
        FieldMetadata(alias="isActive"),
        pydantic.Field(alias="isActive", description="Whether the passenger is active."),
    ]
    last_name: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="lastName"),
        pydantic.Field(alias="lastName", description="Last name of the passenger."),
    ]
    special_instructions: typing_extensions.Annotated[
        typing.Optional[RidershipPassengerSpecialInstructionsObjectResponseBody],
        FieldMetadata(alias="specialInstructions"),
        pydantic.Field(alias="specialInstructions"),
    ] = None
    updated_at_time: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="updatedAtTime"),
        pydantic.Field(
            alias="updatedAtTime", description="The time the passenger was last updated in RFC 3339 format."
        ),
    ]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .route_settings_request_body_route_completion_condition import RouteSettingsRequestBodyRouteCompletionCondition
from .route_settings_request_body_route_starting_condition import RouteSettingsRequestBodyRouteStartingCondition
from .route_settings_request_body_sequencing_method import RouteSettingsRequestBodySequencingMethod


class RouteSettingsRequestBody(UniversalBaseModel):
    """
    An optional dictionary, only necessary to override the defaults for route start and end conditions.
    """

    route_completion_condition: typing_extensions.Annotated[
        typing.Optional[RouteSettingsRequestBodyRouteCompletionCondition],
        FieldMetadata(alias="routeCompletionCondition"),
        pydantic.Field(
            alias="routeCompletionCondition",
            description="Defaults to 'arriveLastStop' which ends the route upon arriving at the final stop. The condition 'departLastStop' \nends the route upon departing the last stop. If 'arriveLastStop' is set, then the departure time of the final stop should not be set.  Valid values: `arriveLastStop`, `departLastStop`",
        ),
    ] = None
    route_starting_condition: typing_extensions.Annotated[
        typing.Optional[RouteSettingsRequestBodyRouteStartingCondition],
        FieldMetadata(alias="routeStartingCondition"),
        pydantic.Field(
            alias="routeStartingCondition",
            description="Defaults to 'departFirstStop' which starts the route upon departing the first stop in the route.\n The condition 'arriveFirstStop' starts the route upon arriving at the first stop in the route. If 'departFirstStop' is set,\nthe arrival time of the first stop should not be set.  Valid values: `departFirstStop`, `arriveFirstStop`",
        ),
    ] = None
    sequencing_method: typing_extensions.Annotated[
        typing.Optional[RouteSettingsRequestBodySequencingMethod],
        FieldMetadata(alias="sequencingMethod"),
        pydantic.Field(
            alias="sequencingMethod",
            description="Determines how stops are sequenced on the route. 'scheduledArrivalTime' sequences stops by their scheduled arrival times (default). 'manual' allows custom sequencing via stop.sequenceNumber. 'unknown' indicates the method is not specified.  Valid values: `unknown`, `scheduledArrivalTime`, `manual`",
        ),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

GetDriversVehicleAssignmentsRequestDriverActivationStatus = typing.Union[
    typing.Literal["active", "deactivated"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

HosViolationTriggerDetailsObjectRequestBodyViolation = typing.Union[
    typing.Literal[
        "CaliforniaMealbreakMissed",
        "CycleHoursOn",
        "DailyDrivingHours",
        "DailyOnDutyHours",
        "Invalid",
        "RestbreakMissed",
        "ShiftDrivingHours",
        "ShiftHours",
        "ShiftOnDutyHours",
        "UnsubmittedLogs",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import typing

VoiceCoachingSettingsObjectResponseBodyLanguage = typing.Union[
    typing.Literal["english", "spanish", "dutch", "englishUk", "italian", "french", "german"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .ridership_passenger_identifier_object_response_body_status import (
    RidershipPassengerIdentifierObjectResponseBodyStatus,
)
from .ridership_passenger_identifier_object_response_body_type import RidershipPassengerIdentifierObjectResponseBodyType


class RidershipPassengerIdentifierObjectResponseBody(UniversalBaseModel):
    """
    An identifier associated with a passenger.
    """

    id: str = pydantic.Field()
    """
    The unique Samsara ID of the identifier.
    """

    status: RidershipPassengerIdentifierObjectResponseBodyStatus = pydantic.Field()
    """
    The status of the identifier.  Valid values: `active`, `inactive`, `unknown`
    """

    type: RidershipPassengerIdentifierObjectResponseBodyType = pydantic.Field()
    """
    The type of identifier.  Valid values: `rfid`, `unknown`
    """

    value: str = pydantic.Field()
    """
    The identifier value.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .camera_details_response_response_body import CameraDetailsResponseResponseBody
from .gateway_details_response_response_body import GatewayDetailsResponseResponseBody


class HealthDetailsResponseResponseBody(UniversalBaseModel):
    """
    Detailed health related metadata for the device.
    """

    camera_details: typing_extensions.Annotated[
        typing.Optional[CameraDetailsResponseResponseBody],
        FieldMetadata(alias="cameraDetails"),
        pydantic.Field(alias="cameraDetails"),
    ] = None
    gateway_details: typing_extensions.Annotated[
        typing.Optional[GatewayDetailsResponseResponseBody],
        FieldMetadata(alias="gatewayDetails"),
        pydantic.Field(alias="gatewayDetails"),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .minimal_route_stop_audit_logs_response_body_state import MinimalRouteStopAuditLogsResponseBodyState


class MinimalRouteStopAuditLogsResponseBody(UniversalBaseModel):
    """
    A single route stop for a route.
    """

    actual_arrival_time: typing_extensions.Annotated[
        typing.Optional[dt.datetime],
        FieldMetadata(alias="actualArrivalTime"),
        pydantic.Field(
            alias="actualArrivalTime",
            description="Actual arrival time, if it exists, for the route stop in RFC 3339 format.",
        ),
    ] = None
    actual_departure_time: typing_extensions.Annotated[
        typing.Optional[dt.datetime],
        FieldMetadata(alias="actualDepartureTime"),
        pydantic.Field(
            alias="actualDepartureTime",
            description="Actual departure time, if it exists, for the route stop in RFC 3339 format.",
        ),
    ] = None
    en_route_time: typing_extensions.Annotated[
        typing.Optional[dt.datetime],
        FieldMetadata(alias="enRouteTime"),
        pydantic.Field(alias="enRouteTime", description="The time the stop became en-route, in RFC 3339 format."),
    ] = None
    eta: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    Estimated time of arrival, if this stop is currently en-route, in RFC 3339 format.
    """

    external_ids: typing_extensions.Annotated[
        typing.Optional[typing.Dict[str, str]],
        FieldMetadata(alias="externalIds"),
        pydantic.Field(alias="externalIds", description="A map of external ids"),
    ] = None
    id: str = pydantic.Field()
    """
    Unique identifier for the route stop.
    """

    live_sharing_url: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="liveSharingUrl"),
        pydantic.Field(alias="liveSharingUrl", description="The shareable url of the stop's current status."),
    ] = None
    scheduled_arrival_time: typing_extensions.Annotated[
        typing.Optional[dt.datetime],
        FieldMetadata(alias="scheduledArrivalTime"),
        pydantic.Field(
            alias="scheduledArrivalTime",
            description="Scheduled arrival time, if it exists, for the stop in RFC 3339 format. If it does not exist, and this field was changed in the update, it will be an empty string.",
        ),
    ] = None
    scheduled_departure_time: typing_extensions.Annotated[
        typing.Optional[dt.datetime],
        FieldMetadata(alias="scheduledDepartureTime"),
        pydantic.Field(
            alias="scheduledDepartureTime",
            description="Scheduled departure time, if it exists, for the stop in RFC 3339 format. If it does not exist, and this field was changed in the update, it will be an empty string.",
        ),
    ] = None
    skipped_time: typing_extensions.Annotated[
        typing.Optional[dt.datetime],
        FieldMetadata(alias="skippedTime"),
        pydantic.Field(
            alias="skippedTime", description="Skipped time, if it exists, for the route stop in RFC 3339 format."
        ),
    ] = None
    state: typing.Optional[MinimalRouteStopAuditLogsResponseBodyState] = pydantic.Field(default=None)
    """
    The current state of the route stop.  Valid values: `unassigned`, `scheduled`, `en route`, `skipped`, `arrived`, `departed`
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.form_submission_request_approval_details_request_body import (
    FormSubmissionRequestApprovalDetailsRequestBody,
)
from ..types.form_submission_request_assigned_to_request_body import FormSubmissionRequestAssignedToRequestBody
from ..types.form_submission_request_field_input_object_request_body import (
    FormSubmissionRequestFieldInputObjectRequestBody,
)
from ..types.form_submissions_get_form_submissions_pdf_exports_response_body import (
    FormSubmissionsGetFormSubmissionsPdfExportsResponseBody,
)
from ..types.form_submissions_get_form_submissions_response_body import FormSubmissionsGetFormSubmissionsResponseBody
from ..types.form_submissions_get_form_submissions_stream_response_body import (
    FormSubmissionsGetFormSubmissionsStreamResponseBody,
)
from ..types.form_submissions_patch_form_submission_response_body import FormSubmissionsPatchFormSubmissionResponseBody
from ..types.form_submissions_post_form_submission_response_body import FormSubmissionsPostFormSubmissionResponseBody
from ..types.form_submissions_post_form_submissions_pdf_exports_response_body import (
    FormSubmissionsPostFormSubmissionsPdfExportsResponseBody,
)
from ..types.form_template_request_object_request_body import FormTemplateRequestObjectRequestBody
from ..types.form_templates_get_form_templates_response_body import FormTemplatesGetFormTemplatesResponseBody
from .raw_client import AsyncRawFormsClient, RawFormsClient
from .types.form_submissions_patch_form_submission_request_body_status import (
    FormSubmissionsPatchFormSubmissionRequestBodyStatus,
)
from .types.form_submissions_post_form_submission_request_body_status import (
    FormSubmissionsPostFormSubmissionRequestBodyStatus,
)

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class FormsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawFormsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawFormsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawFormsClient
        """
        return self._raw_client

    def get_form_submissions(
        self,
        *,
        ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        include: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> FormSubmissionsGetFormSubmissionsResponseBody:
        """
        Returns all form submissions data for the specified list of IDs.

         <b>Rate limit:</b> 5 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Read Form Submissions** under the Forms category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A comma-separated list containing up to 100 form submission IDs to filter on. Can be either a unique Samsara ID or an [external ID](https://developers.samsara.com/docs/external-ids) for the form submission.

        include : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A comma-separated list of strings indicating whether to return additional information. Valid values: `externalIds`

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        FormSubmissionsGetFormSubmissionsResponseBody
            OK response.

        Examples
        --------
        from samsara import Samsara

        client = Samsara(
            token="YOUR_TOKEN",
        )
        client.forms.get_form_submissions()
        """
        _response = self._raw_client.get_form_submissions(ids=ids, include=include, request_options=request_options)
        return _response.data

    def post_form_submission(
        self,
        *,
        form_template: FormTemplateRequestObjectRequestBody,
        status: FormSubmissionsPostFormSubmissionRequestBodyStatus,
        assigned_to: typing.Optional[FormSubmissionRequestAssignedToRequestBody] = OMIT,
        due_at_time: typing.Optional[dt.datetime] = OMIT,
        fields: typing.Optional[typing.Sequence[FormSubmissionRequestFieldInputObjectRequestBody]] = OMIT,
        is_required: typing.Optional[bool] = OMIT,
        route_stop_id: typing.Optional[str] = OMIT,
        title: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> FormSubmissionsPostFormSubmissionResponseBody:
        """
        Creates a form submission and assigns it to an individual worker. This endpoint can be used to create an empty or partially complete form submission and assign it to a worker.

         <b>Rate limit:</b> 100 requests/min (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Write Form Submissions** under the Forms category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        form_template : FormTemplateRequestObjectRequestBody

        status : FormSubmissionsPostFormSubmissionRequestBodyStatus
            Status of the form submission.  Valid values: `notStarted`

        assigned_to : typing.Optional[FormSubmissionRequestAssignedToRequestBody]

        due_at_time : typing.Optional[dt.datetime]
            Due date of the form submission. UTC timestamp in RFC 3339 format.

        fields : typing.Optional[typing.Sequence[FormSubmissionRequestFieldInputObjectRequestBody]]
            List of field inputs in a form submission.

        is_required : typing.Optional[bool]
            Indicates whether the worker is required to complete this form or not at a specific route stop. Defaults to `true` if the form is assigned to a user or driver. When true, the worker cannot depart the route stop until this form submission is `submitted`.

        route_stop_id : typing.Optional[str]
            ID of the route stop the form submission is assigned to. Must be a unique Samsara ID.

        title : typing.Optional[str]
            Title of the form submission.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        FormSubmissionsPostFormSubmissionResponseBody
            OK response.

        Examples
        --------
        from samsara import FormTemplateRequestObjectRequestBody, Samsara

        client = Samsara(
            token="YOUR_TOKEN",
        )
        client.forms.post_form_submission(
            form_template=FormTemplateRequestObjectRequestBody(
                id="9814a1fa-f0c6-408b-bf85-51dc3bc71ac7",
                revision_id="1214a1fa-f0c6-408b-bf85-51dc3bc71ac7",
            ),
            status="notStarted",
        )
        """
        _response = self._raw_client.post_form_submission(
            form_template=form_template,
            status=status,
            assigned_to=assigned_to,
            due_at_time=due_at_time,
            fields=fields,
            is_required=is_required,
            route_stop_id=route_stop_id,
            title=title,
            request_options=request_options,
        )
        return _response.data

    def patch_form_submission(
        self,
        *,
        id: str,
        approval_details: typing.Optional[FormSubmissionRequestApprovalDetailsRequestBody] = OMIT,
        assigned_to: typing.Optional[FormSubmissionRequestAssignedToRequestBody] = OMIT,
        due_at_time: typing.Optional[dt.datetime] = OMIT,
        is_required: typing.Optional[bool] = OMIT,
        route_stop_id: typing.Optional[str] = OMIT,
        status: typing.Optional[FormSubmissionsPatchFormSubmissionRequestBodyStatus] = OMIT,
        title: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> FormSubmissionsPatchFormSubmissionResponseBody:
        """
        Updates an instance of a form submission.

         <b>Rate limit:</b> 100 requests/min (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Write Form Submissions** under the Forms category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        id : str
            ID of the form submission.

        approval_details : typing.Optional[FormSubmissionRequestApprovalDetailsRequestBody]

        assigned_to : typing.Optional[FormSubmissionRequestAssignedToRequestBody]

        due_at_time : typing.Optional[dt.datetime]
            Due date of the form submission. UTC timestamp in RFC 3339 format.

        is_required : typing.Optional[bool]
            Indicates whether the worker is required to complete this form or not at a specific route stop. Defaults to `true` if the form is assigned to a user or driver. When true, the worker cannot depart the route stop until this form submission is `submitted`.

        route_stop_id : typing.Optional[str]
            ID of the route stop the form submission is assigned to. Must be a unique Samsara ID.

        status : typing.Optional[FormSubmissionsPatchFormSubmissionRequestBodyStatus]
            Status of the form submission.  Valid values: `notStarted`, `archived`, `inProgress`, `changesRequested`, `approved`

        title : typing.Optional[str]
            Title of the form submission.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        FormSubmissionsPatchFormSubmissionResponseBody
            OK response.

        Examples
        --------
        from samsara import Samsara

        client = Samsara(
            token="YOUR_TOKEN",
        )
        client.forms.patch_form_submission(
            id="9814a1fa-f0c6-408b-bf85-51dc3bc71ac8",
            status="notStarted",
        )
        """
        _response = self._raw_client.patch_form_submission(
            id=id,
            approval_details=approval_details,
            assigned_to=assigned_to,
            due_at_time=due_at_time,
            is_required=is_required,
            route_stop_id=route_stop_id,
            status=status,
            title=title,
            request_options=request_options,
        )
        return _response.data

    def get_form_submissions_pdf_exports(
        self, *, pdf_id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> FormSubmissionsGetFormSubmissionsPdfExportsResponseBody:
        """
        Returns a PDF export that has already been generated for a form submission.

         <b>Rate limit:</b> 100 requests/min (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Read Form Submissions** under the Forms category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        pdf_id : str
            ID of the form submission PDF export.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        FormSubmissionsGetFormSubmissionsPdfExportsResponseBody
            OK response.

        Examples
        --------
        from samsara import Samsara

        client = Samsara(
            token="YOUR_TOKEN",
        )
        client.forms.get_form_submissions_pdf_exports(
            pdf_id="pdfId",
        )
        """
        _response = self._raw_client.get_form_submissions_pdf_exports(pdf_id=pdf_id, request_options=request_options)
        return _response.data

    def post_form_submissions_pdf_exports(
        self, *, id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> FormSubmissionsPostFormSubmissionsPdfExportsResponseBody:
        """
        Creates a PDF export for an existing form submission.

         <b>Rate limit:</b> 100 requests/min (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Write Form Submissions** under the Forms category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        id : str
            ID of the form submission to create a PDF export from.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        FormSubmissionsPostFormSubmissionsPdfExportsResponseBody
            Accepted response.

        Examples
        --------
        from samsara import Samsara

        client = Samsara(
            token="YOUR_TOKEN",
        )
        client.forms.post_form_submissions_pdf_exports(
            id="id",
        )
        """
        _response = self._raw_client.post_form_submissions_pdf_exports(id=id, request_options=request_options)
        return _response.data

    def get_form_submissions_stream(
        self,
        *,
        start_time: str,
        end_time: typing.Optional[str] = None,
        after: typing.Optional[str] = None,
        form_template_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        user_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        driver_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        include: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        assigned_to_route_stop_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> FormSubmissionsGetFormSubmissionsStreamResponseBody:
        """
        Returns all form submissions data that has been created or modified for your organization based on the time parameters passed in. Results are paginated and are sorted by last modified date. If you include an endTime, the endpoint will return data up until that point (exclusive). If you don’t include an endTime, you can continue to poll the API real-time with the pagination cursor that gets returned on every call.

         <b>Rate limit:</b> 5 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Read Form Submissions** under the Forms category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        start_time : str
            A start time in RFC 3339 format. Millisecond precision and timezones are supported. Value is compared against `updatedAtTime`. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        end_time : typing.Optional[str]
            An end time in RFC 3339 format. Defaults to now if not provided. Millisecond precision and timezones are supported. Value is compared against `updatedAtTime`. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        after : typing.Optional[str]
             If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        form_template_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A comma-separated list containing up to 50 template IDs to filter data to.

        user_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A comma-separated list containing up to 50 user IDs to filter data to.

        driver_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A comma-separated list containing up to 50 user IDs to filter data to.

        include : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A comma-separated list of strings indicating whether to return additional information. Valid values: `externalIds`

        assigned_to_route_stop_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A comma-separated list containing up to 50 route stop IDs to filter data to.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        FormSubmissionsGetFormSubmissionsStreamResponseBody
            OK response.

        Examples
        --------
        from samsara import Samsara

        client = Samsara(
            token="YOUR_TOKEN",
        )
        client.forms.get_form_submissions_stream(
            start_time="startTime",
        )
        """
        _response = self._raw_client.get_form_submissions_stream(
            start_time=start_time,
            end_time=end_time,
            after=after,
            form_template_ids=form_template_ids,
            user_ids=user_ids,
            driver_ids=driver_ids,
            include=include,
            assigned_to_route_stop_ids=assigned_to_route_stop_ids,
            request_options=request_options,
        )
        return _response.data

    def get_form_templates(
        self,
        *,
        ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        after: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> FormTemplatesGetFormTemplatesResponseBody:
        """
        Returns a list of the organization's form templates for the specified list of IDs. If no IDs are provided, all form templates for the organization will be returned.

         <b>Rate limit:</b> 5 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Read Form Submissions** under the Forms category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A comma-separated list containing up to 100 template IDs to filter on.

        after : typing.Optional[str]
             If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        FormTemplatesGetFormTemplatesResponseBody
            OK response.

        Examples
        --------
        from samsara import Samsara

        client = Samsara(
            token="YOUR_TOKEN",
        )
        client.forms.get_form_templates()
        """
        _response = self._raw_client.get_form_templates(ids=ids, after=after, request_options=request_options)
        return _response.data


class AsyncFormsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawFormsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawFormsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawFormsClient
        """
        return self._raw_client

    async def get_form_submissions(
        self,
        *,
        ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        include: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> FormSubmissionsGetFormSubmissionsResponseBody:
        """
        Returns all form submissions data for the specified list of IDs.

         <b>Rate limit:</b> 5 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Read Form Submissions** under the Forms category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A comma-separated list containing up to 100 form submission IDs to filter on. Can be either a unique Samsara ID or an [external ID](https://developers.samsara.com/docs/external-ids) for the form submission.

        include : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A comma-separated list of strings indicating whether to return additional information. Valid values: `externalIds`

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        FormSubmissionsGetFormSubmissionsResponseBody
            OK response.

        Examples
        --------
        import asyncio

        from samsara import AsyncSamsara

        client = AsyncSamsara(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.forms.get_form_submissions()


        asyncio.run(main())
        """
        _response = await self._raw_client.get_form_submissions(
            ids=ids, include=include, request_options=request_options
        )
        return _response.data

    async def post_form_submission(
        self,
        *,
        form_template: FormTemplateRequestObjectRequestBody,
        status: FormSubmissionsPostFormSubmissionRequestBodyStatus,
        assigned_to: typing.Optional[FormSubmissionRequestAssignedToRequestBody] = OMIT,
        due_at_time: typing.Optional[dt.datetime] = OMIT,
        fields: typing.Optional[typing.Sequence[FormSubmissionRequestFieldInputObjectRequestBody]] = OMIT,
        is_required: typing.Optional[bool] = OMIT,
        route_stop_id: typing.Optional[str] = OMIT,
        title: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> FormSubmissionsPostFormSubmissionResponseBody:
        """
        Creates a form submission and assigns it to an individual worker. This endpoint can be used to create an empty or partially complete form submission and assign it to a worker.

         <b>Rate limit:</b> 100 requests/min (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Write Form Submissions** under the Forms category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        form_template : FormTemplateRequestObjectRequestBody

        status : FormSubmissionsPostFormSubmissionRequestBodyStatus
            Status of the form submission.  Valid values: `notStarted`

        assigned_to : typing.Optional[FormSubmissionRequestAssignedToRequestBody]

        due_at_time : typing.Optional[dt.datetime]
            Due date of the form submission. UTC timestamp in RFC 3339 format.

        fields : typing.Optional[typing.Sequence[FormSubmissionRequestFieldInputObjectRequestBody]]
            List of field inputs in a form submission.

        is_required : typing.Optional[bool]
            Indicates whether the worker is required to complete this form or not at a specific route stop. Defaults to `true` if the form is assigned to a user or driver. When true, the worker cannot depart the route stop until this form submission is `submitted`.

        route_stop_id : typing.Optional[str]
            ID of the route stop the form submission is assigned to. Must be a unique Samsara ID.

        title : typing.Optional[str]
            Title of the form submission.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        FormSubmissionsPostFormSubmissionResponseBody
            OK response.

        Examples
        --------
        import asyncio

        from samsara import AsyncSamsara, FormTemplateRequestObjectRequestBody

        client = AsyncSamsara(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.forms.post_form_submission(
                form_template=FormTemplateRequestObjectRequestBody(
                    id="9814a1fa-f0c6-408b-bf85-51dc3bc71ac7",
                    revision_id="1214a1fa-f0c6-408b-bf85-51dc3bc71ac7",
                ),
                status="notStarted",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.post_form_submission(
            form_template=form_template,
            status=status,
            assigned_to=assigned_to,
            due_at_time=due_at_time,
            fields=fields,
            is_required=is_required,
            route_stop_id=route_stop_id,
            title=title,
            request_options=request_options,
        )
        return _response.data

    async def patch_form_submission(
        self,
        *,
        id: str,
        approval_details: typing.Optional[FormSubmissionRequestApprovalDetailsRequestBody] = OMIT,
        assigned_to: typing.Optional[FormSubmissionRequestAssignedToRequestBody] = OMIT,
        due_at_time: typing.Optional[dt.datetime] = OMIT,
        is_required: typing.Optional[bool] = OMIT,
        route_stop_id: typing.Optional[str] = OMIT,
        status: typing.Optional[FormSubmissionsPatchFormSubmissionRequestBodyStatus] = OMIT,
        title: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> FormSubmissionsPatchFormSubmissionResponseBody:
        """
        Updates an instance of a form submission.

         <b>Rate limit:</b> 100 requests/min (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Write Form Submissions** under the Forms category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        id : str
            ID of the form submission.

        approval_details : typing.Optional[FormSubmissionRequestApprovalDetailsRequestBody]

        assigned_to : typing.Optional[FormSubmissionRequestAssignedToRequestBody]

        due_at_time : typing.Optional[dt.datetime]
            Due date of the form submission. UTC timestamp in RFC 3339 format.

        is_required : typing.Optional[bool]
            Indicates whether the worker is required to complete this form or not at a specific route stop. Defaults to `true` if the form is assigned to a user or driver. When true, the worker cannot depart the route stop until this form submission is `submitted`.

        route_stop_id : typing.Optional[str]
            ID of the route stop the form submission is assigned to. Must be a unique Samsara ID.

        status : typing.Optional[FormSubmissionsPatchFormSubmissionRequestBodyStatus]
            Status of the form submission.  Valid values: `notStarted`, `archived`, `inProgress`, `changesRequested`, `approved`

        title : typing.Optional[str]
            Title of the form submission.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        FormSubmissionsPatchFormSubmissionResponseBody
            OK response.

        Examples
        --------
        import asyncio

        from samsara import AsyncSamsara

        client = AsyncSamsara(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.forms.patch_form_submission(
                id="9814a1fa-f0c6-408b-bf85-51dc3bc71ac8",
                status="notStarted",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.patch_form_submission(
            id=id,
            approval_details=approval_details,
            assigned_to=assigned_to,
            due_at_time=due_at_time,
            is_required=is_required,
            route_stop_id=route_stop_id,
            status=status,
            title=title,
            request_options=request_options,
        )
        return _response.data

    async def get_form_submissions_pdf_exports(
        self, *, pdf_id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> FormSubmissionsGetFormSubmissionsPdfExportsResponseBody:
        """
        Returns a PDF export that has already been generated for a form submission.

         <b>Rate limit:</b> 100 requests/min (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Read Form Submissions** under the Forms category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        pdf_id : str
            ID of the form submission PDF export.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        FormSubmissionsGetFormSubmissionsPdfExportsResponseBody
            OK response.

        Examples
        --------
        import asyncio

        from samsara import AsyncSamsara

        client = AsyncSamsara(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.forms.get_form_submissions_pdf_exports(
                pdf_id="pdfId",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_form_submissions_pdf_exports(
            pdf_id=pdf_id, request_options=request_options
        )
        return _response.data

    async def post_form_submissions_pdf_exports(
        self, *, id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> FormSubmissionsPostFormSubmissionsPdfExportsResponseBody:
        """
        Creates a PDF export for an existing form submission.

         <b>Rate limit:</b> 100 requests/min (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Write Form Submissions** under the Forms category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        id : str
            ID of the form submission to create a PDF export from.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        FormSubmissionsPostFormSubmissionsPdfExportsResponseBody
            Accepted response.

        Examples
        --------
        import asyncio

        from samsara import AsyncSamsara

        client = AsyncSamsara(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.forms.post_form_submissions_pdf_exports(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.post_form_submissions_pdf_exports(id=id, request_options=request_options)
        return _response.data

    async def get_form_submissions_stream(
        self,
        *,
        start_time: str,
        end_time: typing.Optional[str] = None,
        after: typing.Optional[str] = None,
        form_template_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        user_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        driver_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        include: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        assigned_to_route_stop_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> FormSubmissionsGetFormSubmissionsStreamResponseBody:
        """
        Returns all form submissions data that has been created or modified for your organization based on the time parameters passed in. Results are paginated and are sorted by last modified date. If you include an endTime, the endpoint will return data up until that point (exclusive). If you don’t include an endTime, you can continue to poll the API real-time with the pagination cursor that gets returned on every call.

         <b>Rate limit:</b> 5 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Read Form Submissions** under the Forms category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        start_time : str
            A start time in RFC 3339 format. Millisecond precision and timezones are supported. Value is compared against `updatedAtTime`. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        end_time : typing.Optional[str]
            An end time in RFC 3339 format. Defaults to now if not provided. Millisecond precision and timezones are supported. Value is compared against `updatedAtTime`. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        after : typing.Optional[str]
             If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        form_template_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A comma-separated list containing up to 50 template IDs to filter data to.

        user_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A comma-separated list containing up to 50 user IDs to filter data to.

        driver_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A comma-separated list containing up to 50 user IDs to filter data to.

        include : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A comma-separated list of strings indicating whether to return additional information. Valid values: `externalIds`

        assigned_to_route_stop_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A comma-separated list containing up to 50 route stop IDs to filter data to.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        FormSubmissionsGetFormSubmissionsStreamResponseBody
            OK response.

        Examples
        --------
        import asyncio

        from samsara import AsyncSamsara

        client = AsyncSamsara(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.forms.get_form_submissions_stream(
                start_time="startTime",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_form_submissions_stream(
            start_time=start_time,
            end_time=end_time,
            after=after,
            form_template_ids=form_template_ids,
            user_ids=user_ids,
            driver_ids=driver_ids,
            include=include,
            assigned_to_route_stop_ids=assigned_to_route_stop_ids,
            request_options=request_options,
        )
        return _response.data

    async def get_form_templates(
        self,
        *,
        ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        after: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> FormTemplatesGetFormTemplatesResponseBody:
        """
        Returns a list of the organization's form templates for the specified list of IDs. If no IDs are provided, all form templates for the organization will be returned.

         <b>Rate limit:</b> 5 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Read Form Submissions** under the Forms category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A comma-separated list containing up to 100 template IDs to filter on.

        after : typing.Optional[str]
             If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        FormTemplatesGetFormTemplatesResponseBody
            OK response.

        Examples
        --------
        import asyncio

        from samsara import AsyncSamsara

        client = AsyncSamsara(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.forms.get_form_templates()


        asyncio.run(main())
        """
        _response = await self._raw_client.get_form_templates(ids=ids, after=after, request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .health_reason_response_response_body_health_reason_code import HealthReasonResponseResponseBodyHealthReasonCode


class HealthReasonResponseResponseBody(UniversalBaseModel):
    """
    Information about an active health reason affecting the device.
    """

    health_reason_code: typing_extensions.Annotated[
        typing.Optional[HealthReasonResponseResponseBodyHealthReasonCode],
        FieldMetadata(alias="healthReasonCode"),
        pydantic.Field(
            alias="healthReasonCode",
            description="The type of health reason detected.  Valid values: `assetUnplugged`, `cameraMisaligned`, `dataPending`, `healthy`, `inwardCameraObstruction`, `irregularRecording`, `lowDeviceBattery`, `missingEldDiagnostics`, `missingVin`, `needsReplacement`, `newlyInstalledDevice`, `noGpsSignal`, `notDetected`, `outwardCameraObstruction`, `recordingTimeRequired`, `unknown`, `vgUnplugged`",
        ),
    ] = None
    start_time: typing_extensions.Annotated[
        typing.Optional[dt.datetime],
        FieldMetadata(alias="startTime"),
        pydantic.Field(
            alias="startTime", description="The timestamp when this health reason began, in RFC3339 format."
        ),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .forms_asset_value_object_response_body import FormsAssetValueObjectResponseBody
from .forms_check_boxes_value_object_response_body import FormsCheckBoxesValueObjectResponseBody
from .forms_date_time_value_object_response_body import FormsDateTimeValueObjectResponseBody
from .forms_field_input_object_response_body_type import FormsFieldInputObjectResponseBodyType
from .forms_geofence_value_object_response_body import FormsGeofenceValueObjectResponseBody
from .forms_issue_created_by_field_object_response_body import FormsIssueCreatedByFieldObjectResponseBody
from .forms_media_record_object_response_body import FormsMediaRecordObjectResponseBody
from .forms_media_value_object_response_body import FormsMediaValueObjectResponseBody
from .forms_multiple_choice_value_object_response_body import FormsMultipleChoiceValueObjectResponseBody
from .forms_number_value_object_response_body import FormsNumberValueObjectResponseBody
from .forms_person_value_object_response_body import FormsPersonValueObjectResponseBody
from .forms_signature_value_object_response_body import FormsSignatureValueObjectResponseBody
from .forms_table_value_object_response_body import FormsTableValueObjectResponseBody
from .forms_text_value_object_response_body import FormsTextValueObjectResponseBody


class FormsFieldInputObjectResponseBody(UniversalBaseModel):
    """
    Forms input field object.
    """

    asset_value: typing_extensions.Annotated[
        typing.Optional[FormsAssetValueObjectResponseBody],
        FieldMetadata(alias="assetValue"),
        pydantic.Field(alias="assetValue"),
    ] = None
    check_boxes_value: typing_extensions.Annotated[
        typing.Optional[FormsCheckBoxesValueObjectResponseBody],
        FieldMetadata(alias="checkBoxesValue"),
        pydantic.Field(alias="checkBoxesValue"),
    ] = None
    date_time_value: typing_extensions.Annotated[
        typing.Optional[FormsDateTimeValueObjectResponseBody],
        FieldMetadata(alias="dateTimeValue"),
        pydantic.Field(alias="dateTimeValue"),
    ] = None
    geofence_value: typing_extensions.Annotated[
        typing.Optional[FormsGeofenceValueObjectResponseBody],
        FieldMetadata(alias="geofenceValue"),
        pydantic.Field(alias="geofenceValue"),
    ] = None
    id: str = pydantic.Field()
    """
    ID of the forms input field object.
    """

    issue: typing.Optional[FormsIssueCreatedByFieldObjectResponseBody] = None
    label: typing.Optional[str] = pydantic.Field(default=None)
    """
    Forms input field label.
    """

    media_list: typing_extensions.Annotated[
        typing.Optional[typing.List[FormsMediaRecordObjectResponseBody]],
        FieldMetadata(alias="mediaList"),
        pydantic.Field(alias="mediaList", description="List of forms media record objects."),
    ] = None
    media_value: typing_extensions.Annotated[
        typing.Optional[FormsMediaValueObjectResponseBody],
        FieldMetadata(alias="mediaValue"),
        pydantic.Field(alias="mediaValue"),
    ] = None
    multiple_choice_value: typing_extensions.Annotated[
        typing.Optional[FormsMultipleChoiceValueObjectResponseBody],
        FieldMetadata(alias="multipleChoiceValue"),
        pydantic.Field(alias="multipleChoiceValue"),
    ] = None
    note: typing.Optional[str] = pydantic.Field(default=None)
    """
    A note attached to the field input.
    """

    number_value: typing_extensions.Annotated[
        typing.Optional[FormsNumberValueObjectResponseBody],
        FieldMetadata(alias="numberValue"),
        pydantic.Field(alias="numberValue"),
    ] = None
    person_value: typing_extensions.Annotated[
        typing.Optional[FormsPersonValueObjectResponseBody],
        FieldMetadata(alias="personValue"),
        pydantic.Field(alias="personValue"),
    ] = None
    signature_value: typing_extensions.Annotated[
        typing.Optional[FormsSignatureValueObjectResponseBody],
        FieldMetadata(alias="signatureValue"),
        pydantic.Field(alias="signatureValue"),
    ] = None
    table_value: typing_extensions.Annotated[
        typing.Optional[FormsTableValueObjectResponseBody],
        FieldMetadata(alias="tableValue"),
        pydantic.Field(alias="tableValue"),
    ] = None
    text_value: typing_extensions.Annotated[
        typing.Optional[FormsTextValueObjectResponseBody],
        FieldMetadata(alias="textValue"),
        pydantic.Field(alias="textValue"),
    ] = None
    type: FormsFieldInputObjectResponseBodyType = pydantic.Field()
    """
    Type of the field.  Valid values: `number`, `text`, `multiple_choice`, `check_boxes`, `datetime`, `signature`, `media`, `asset`, `table`, `person`, `geofence`
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

SpeedingIntervalResponseBodySeverityLevel = typing.Union[
    typing.Literal["light", "moderate", "heavy", "severe"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .conditional_field_section_object_response_body import ConditionalFieldSectionObjectResponseBody
from .field_types_object_response_body import FieldTypesObjectResponseBody


class GetDocumentTypeResponseObjectResponseBody(UniversalBaseModel):
    conditional_field_sections: typing_extensions.Annotated[
        typing.Optional[typing.List[ConditionalFieldSectionObjectResponseBody]],
        FieldMetadata(alias="conditionalFieldSections"),
        pydantic.Field(
            alias="conditionalFieldSections", description="List of the document type conditional field sections."
        ),
    ] = None
    field_types: typing_extensions.Annotated[
        typing.Optional[typing.List[FieldTypesObjectResponseBody]],
        FieldMetadata(alias="fieldTypes"),
        pydantic.Field(alias="fieldTypes", description="The fields associated with this document type."),
    ] = None
    id: typing.Optional[str] = pydantic.Field(default=None)
    """
    Universally unique identifier for the document type. This value can be passed in as the documentTypeId when creating a document.
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    Name of the document type.
    """

    org_id: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="orgId"),
        pydantic.Field(alias="orgId", description="ID for the organization this document type belongs to."),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
